package com.example.guitar_music_app.login

import android.content.Intent
import android.graphics.drawable.AnimationDrawable
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.addCallback
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import com.example.guitar_music_app.*
import com.example.guitar_music_app.general.ANTENNA_LOOP
import com.example.guitar_music_app.general.RC_SIGN_IN
import com.example.guitar_music_app.general.SIGNED_IN
import com.example.guitar_music_app.results.ResultActivity
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.common.api.ApiException
import kotlinx.android.synthetic.main.login_fragment.*

class LoginView : Fragment() {
    private lateinit var viewModel : LoginViewModel

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.login_fragment, container, false)
    }

    override fun onStart() {
        super.onStart()
        viewModel = ViewModelProvider(
            this,
            LoginInjector(requireActivity().application).provideUserViewModelFactory())[LoginViewModel::class.java]
        setUpClickListeners()
        observeViewModel()

        viewModel.handleEvent(LoginEvent.OnStart)
    }

     private fun observeViewModel() {
        viewModel.signInStatusText.observe(
            viewLifecycleOwner,
            {
                loginStatusTextView.text = it
            }
        )
         viewModel.signedIn.observe(
             viewLifecycleOwner,
             {
                 if (it) {
                     startResultActivity()
                     //TODO-OnBackPressedDispatcher nahradit, neni to "onBack"
                     requireActivity().onBackPressedDispatcher.addCallback(this) {
                         startResultActivity()
                     }
                 }
                 else if (loginStatusTextView.text  == SIGNED_IN) {
                     startSignInFlow()
                 }
             }
         )

         viewModel.signInStatusText.observe(
             viewLifecycleOwner,
             {
                 if (loginStatusTextView.text == SIGNED_IN) {
                     startResultActivity()
                 }
             }
         )

        viewModel.authButtonText.observe(
            viewLifecycleOwner,
            {
                authButton.text = it
            }
        )

        viewModel.startAnimation.observe(
            viewLifecycleOwner,
            {
                loginAnimation.setImageResource(
                    resources.getIdentifier(ANTENNA_LOOP, "drawable", activity?.packageName)
                )
                (loginAnimation.drawable as AnimationDrawable).start()
            }
        )

        viewModel.authAttempt.observe(
            viewLifecycleOwner,
            { startSignInFlow() }
        )

        viewModel.antennaDrawable.observe(
            viewLifecycleOwner,
            {
                loginAnimation.setImageResource(
                    resources.getIdentifier(it, "drawable", activity?.packageName)
                )
            }
        )
    }

    private fun startSignInFlow() {
        val gso = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
            .requestIdToken(getString(R.string.default_web_client_id))
            .build()

        val googleSignInClient = GoogleSignIn.getClient(requireActivity(), gso)

        val signInIntent = googleSignInClient.signInIntent
        startActivityForResult(signInIntent, RC_SIGN_IN)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        val task = GoogleSignIn.getSignedInAccountFromIntent(data)
        var token: String? = null

        try {
            val account: GoogleSignInAccount? = task.getResult(ApiException::class.java)

            if (account != null) token = account.idToken

        } catch (exception: Exception) {
            Log.d("Přihlášení", exception.toString())
        }

        viewModel.handleEvent(
            LoginEvent.OnGoogleSignInResult(
                UserLoginResult(
                    requestCode,
                    token
                )
            )
        )
    }

    private fun setUpClickListeners() {
        authButton.setOnClickListener{viewModel.handleEvent(LoginEvent.OnAuthButtonClick)
            }

       btn_hmpg.setOnClickListener{ startResultActivity() }
        requireActivity().onBackPressedDispatcher.addCallback(this) {startResultActivity()}

    }
// Navigace není používaná, protože nefunguje správně s onActivityResult
    private fun startResultActivity() = requireActivity().startActivity(
        Intent(
            activity,
            ResultActivity::class.java
        )
    )


}